<?php

// =============================================================================
// Inline Tweets
//
// Released under the GNU General Public Licence v2
// http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
//
// Please refer all questions/requests to: mdjekic@gmail.com
//
// This is an add-on for WordPress
// http://wordpress.org/
// =============================================================================

// =============================================================================
// This piece of software is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.
// =============================================================================

/*
  Plugin Name: Inline Tweets
  Plugin URI: http://projects.djekic.net/inline-tweets
  Description: Embed latest tweet(s) or a specific tweet in any page, post or text widget and apply your custom styles.
  Version: 1.1.1
  Author: Miloš Đekić
  Author URI: http://milos.djekic.net
 */

// timeout in minutes for caching API responses
define('INLINE_TWEETS_CACHE_TIMEOUT',5);

/**
 * Determines time difference in minutes
 *
 * @param $to current time
 * @param $from starting point in time
 *
 * @return float
 */
function inline_tweets_time_diff_in_minutes($to,$from) {
    return round(abs($to - $from) / 60,2);
}

/**
 * Returns the error HTML entity
 *
 * @param string $error text to be displayed as error
 * @return string HTML
 */
function inline_tweets_inline_error($error = "Error") {
    return '<div class="inline_tweets_error">'.$error.'</div>';
}

/**
 * Embeds tweet into an HTML entity
 *
 * @param $tweet
 * @return string HTML
 */
function inline_tweets_embed_tweet($tweet) {
    // get tweet content
    $content = $tweet['text'];

    // append tweet author
    $content = '<a class="inline_tweets_author" href="http://twitter.com/'.$tweet['user']['screen_name'].'" target="_blank">'."@{$tweet['user']['screen_name']}".'</a> ' . $content;

    // render tags
    foreach($tweet["entities"]["hashtags"] as $tag) {
        $content = str_replace("#{$tag['text']}",'<span class="inline_tweets_tag">'."#{$tag['text']}".'</span>',$content);
    }

    // render URLs
    foreach($tweet["entities"]["urls"] as $url) {
        $content = str_replace($url["url"],'<a class="inline_tweets_url" href="'.$url["expanded_url"].'" target="_blank">'.$url["url"].'</a>',$content);
    }

    // render mentions
    foreach($tweet["entities"]["user_mentions"] as $mention) {
        $content = str_replace("@{$mention['screen_name']}",'<a class="inline_tweets_mention" href="http://twitter.com/'.$mention['screen_name'].'" target="_blank">'."@{$mention['screen_name']}".'</a>',$content);
    }

    // add timestamp
    $content .= ' <span class="inline_tweets_timestamp">' . date('M d',strtotime($tweet['created_at'])) . '</span>';

    // return rendered tweet content
    return $content;
}

/**
 * Performs a curl request to Twitter API to fetch tweet(s)
 *
 * @param $url
 * @return mixed
 */
function inline_tweets_curl($url) {
    // init curl
    $ch = curl_init();

    // parametrize curl calls
    curl_setopt($ch, CURLOPT_URL,$url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

    // @since 1.0.1 - handle SSL issue
    curl_setopt ($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt ($ch, CURLOPT_SSL_VERIFYPEER, 0);

    // get result
    $result = curl_exec($ch);

    // close curl stream
    curl_close($ch);

    // return result
    return $result;
}

/**
 * Request handler intercepts requests to admin.php and checks if any actions
 * should be performed ('inline_tweets_action' parameter is in the request array)
 */
function inline_tweets_request_handler() {
    // check if inline_tweets action is to be performed
    if(empty($_REQUEST['inline_tweets_action'])) return;

    // handle tweet
    $result = inline_tweets_handle($_REQUEST['inline_tweets_action']);

    // send response
    header('Content-type: application/json');
    echo json_encode($result);
    die;
}

/**
 * Handles an inline tweet
 *
 * @param $inline_tweet [account:action:param]
 * @return stdClass data[url,result]
 */
function inline_tweets_handle($inline_tweet) {
    // clear tags
    $inline_tweet = str_replace('[tweet]','',$inline_tweet);
    $inline_tweet = str_replace('[/tweet]','',$inline_tweet);

    // get embed data
    $embed_data = explode(':',$inline_tweet);

    // check data structure
    if(count($embed_data) < 2) return inline_tweets_inline_error("Invalid structure.");

  	// init result
  	$result = new stdClass();
	$result->url = "n/a";
  
    // check action
    switch($embed_data[0]) {
        case 'latest':
            // check structure
            if(count($embed_data) < 3) {
              	$result->html = inline_tweets_inline_error("Invalid structure.");
              	break;
            }

            // determine screen name
            $screen_name = $embed_data[1];

            // check screen name
            if(!preg_match('/^[A-Za-z0-9_]{1,15}$/', $screen_name)) {
                $result->html = inline_tweets_inline_error("Invalid Twitter username.");
                break;
            }

            // determine count
            $count = intval($embed_data[2]);

            // check count
      		if($count == 0) {
        		$result->html = inline_tweets_inline_error("Invalid tweets count.");
				break;
      		}

            // form cache handle
            $result->cache_handle = "inline_tweets_latest_{$screen_name}_{$count}";

            // get cache entry
            $cache_entry = wp_cache_get($result->cache_handle);

            // check if cache entry exists and is young enough
            if($cache_entry !== false && inline_tweets_time_diff_in_minutes(time(),$cache_entry['timestamp']) <= INLINE_TWEETS_CACHE_TIMEOUT) {
                // get data from cache
                $data = $cache_entry['data'];

                // set from cache flag
                $result->from_cache = true;
            }
            else {
                // form URL
                $result->url = "https://api.twitter.com/1/statuses/user_timeline.json"
                    ."?include_entities=true&include_rts=true&screen_name={$screen_name}&count={$count}";

                // get response from Twitter API
                $response = inline_tweets_curl($result->url);

                // decode from JSON to array
                $data = @json_decode($response,true);

                // check data and return inline error if data is false
                if($data == false) {
                    $result->html = inline_tweets_inline_error("Unable to fetch tweet(s)");
                    break;
                }

                // check if there was an API error
                if(isset($data["error"])) {
                    $result->html = inline_tweets_inline_error($data["error"]);
                    break;
                }

                // save to cache
                wp_cache_set($result->cache_handle,array('timestamp' => time(), 'data' => $data));

                // set from cache flag
                $result->from_cache = false;
            }

            // start creating content
            $content = ($count == 1) ? '' : '<ul class="inline_tweets_ul">';

            // go trough all fetched tweets
            foreach($data as $tweet) {
                // input embedded tweet
                $content .= ($count == 1) ? '<div class="inline_tweet">' : '<li class="inline_tweet inline_tweets_li">';
                $content .= inline_tweets_embed_tweet($tweet);
                $content .= ($count == 1) ? '</div>' : '</li>';
            }

            // finish content
            $content .= ($count == 1) ? '' : '</ul>';

            // save created content
            $result->html = $content; break;
        case 'status':
            // determine status id
            $id = ($embed_data[1]);

            // form cache handle
            $result->cache_handle = "inline_tweets_status_{$id}";

            // get cache entry
            $cache_entry = wp_cache_get($result->cache_handle);

            // check if cache entry exists and is young enough
            if($cache_entry !== false && inline_tweets_time_diff_in_minutes(time(),$cache_entry['timestamp']) <= INLINE_TWEETS_CACHE_TIMEOUT) {
                // get data from cache
                $data = $cache_entry['data'];

                // set from cache flag
                $result->from_cache = true;
            }
            else {
                // form URL
                $result->url = "https://api.twitter.com/1/statuses/show.json?id={$id}&include_entities=true";

                // get response from Twitter API
                $response = inline_tweets_curl($result->url);

                // decode from JSON to array
                $data = @json_decode($response,true);

                // check data and return inline error if data is false
                if($data == false) {
                    $result->html = inline_tweets_inline_error("Unable to fetch tweet.");
                    break;
                }

                // check if there was an API error
                if(isset($data["error"])) {
                    $result->html = inline_tweets_inline_error($data["error"]);
                    break;
                }

                // save to cache
                wp_cache_set($result->cache_handle,array('timestamp' => time(), 'data' => $data));

                // set from cache flag
                $result->from_cache = false;
            }

            // save tweet
            $result->html = '<div class="inline_tweet">' . inline_tweets_embed_tweet($data) . '</div>'; break;
        default: $result->html = inline_tweets_inline_error();
    }

    // return result
    return $result;
}

/**
 * Renders posts/pages content by changing placeholders with tweets
 *
 * @param $content content to be rendered
 * @return String rendered content
 */
function inline_tweets_render($content) {
    // define regex for searching for inline tweets
    $regex = '/\[tweet\](.*?)\[\/tweet\]/ism';

    // search for inline tweets
    $count = preg_match_all($regex,$content,$inline_tweets);

    // do nothing if there were no matches
    if($count == 0) return $content;

    // handle all matches
    if(isset($inline_tweets[0])) foreach($inline_tweets[0] as $inline_tweets_data) {
        // wrap inline tweet in placeholder div
        $content = str_replace($inline_tweets_data,'<div class="inline_tweets_placeholder">'.$inline_tweets_data.'</div>',$content);
    }

    // return content
    return $content;
}

/**
 * Loads Inline Tweets script in header
 */
function inline_tweets_load_js_in_header() {
    // load basic css styles
    echo '<style type="text/css">div.inline_tweets_placeholder{text-align: center;}</style>\n';

    // set url param for the loader image
    echo '<script type="text/javascript">var INLINE_TWEETS_BASE_URL = "' . get_bloginfo('siteurl') . '";</script>\n';
  
  	// set url param for the loader image
    echo '<script type="text/javascript">var INLINE_TWEETS_LOADER_URL = "' . plugins_url('/loader.gif',__FILE__) . '";</script>\n';

    // load handling script
    wp_enqueue_script('inline_tweets', plugins_url('/inline_tweets.js',__FILE__));
}

// register request handler
add_action('init', 'inline_tweets_request_handler', 9999);

// add rendering filter for posts/pages
add_filter('the_content','inline_tweets_render');

// add rendering filter for text widgets content
add_filter('widget_text','inline_tweets_render');

// add head action to include inline-tweets script
add_action('wp_head', 'inline_tweets_load_js_in_header');